"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshUA = void 0;
const path_1 = require("path");
const util_1 = require("util");
const child_process_1 = require("child_process");
const semver_1 = __importDefault(require("semver"));
const opLog_1 = require("../opLog/opLog");
const exec = util_1.promisify(child_process_1.exec);
const opLog = opLog_1.getOpLog();
function refreshUA() {
    return __awaiter(this, void 0, void 0, function* () {
        const localPath = process.cwd();
        const ayakashiPath = path_1.resolve(__dirname, "../..");
        try {
            process.chdir(ayakashiPath);
            const pkg = path_1.resolve(ayakashiPath, "node_modules", "user-agents", "package.json");
            const oldVersion = require(pkg).version;
            const latestVersion = (yield exec("npm view user-agents dist-tags.latest")).stdout.trim();
            if (semver_1.default.gte(oldVersion, latestVersion)) {
                opLog.info("User agents are already up to date");
                return;
            }
            const waiter = opLog.waiter("Updating user-agents");
            yield exec("npm update user-agents --legacy-peer-deps");
            delete require.cache[pkg];
            const newVersion = require(pkg).version;
            waiter.succeed(`User agents updated ${oldVersion} -> ${newVersion}`);
            process.chdir(localPath);
        }
        catch (_e) {
            process.chdir(localPath);
            opLog.error("Failed to update user agents. Builtin database will be used");
        }
    });
}
exports.refreshUA = refreshUA;
