"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateAction = void 0;
const mkdirp_1 = __importDefault(require("mkdirp"));
const util_1 = require("util");
const path_1 = require("path");
const fs_1 = require("fs");
const opLog_1 = require("../../opLog/opLog");
const mkdirp = util_1.promisify(mkdirp_1.default);
const writeFile = util_1.promisify(fs_1.writeFile);
const exists = util_1.promisify(fs_1.exists);
function generateAction(directory, name, ts) {
    return __awaiter(this, void 0, void 0, function* () {
        const opLog = opLog_1.getOpLog();
        const ext = ts ? ".ts" : ".js";
        let fileName;
        if (name.indexOf(ext) > -1) {
            fileName = name;
        }
        else {
            fileName = `${name}${ext}`;
        }
        const actionsFolder = path_1.join(directory, "actions");
        const filePath = path_1.join(actionsFolder, fileName);
        if (yield exists(filePath)) {
            opLog.error(`action <${name}> already exists in ${filePath}`);
            return;
        }
        yield mkdirp(actionsFolder);
        const content = ts ? getContentTS(name) : getContent(name);
        yield writeFile(filePath, content);
        opLog.info(`Created <${name}> in ${filePath}`);
    });
}
exports.generateAction = generateAction;
function getContent(name) {
    return (`/**
 * @param {import("@ayakashi/types").IAyakashiInstance} ayakashi
 */
module.exports = function(ayakashi) {
    ayakashi.registerAction("${name}", async function(prop) {
        //prop boilerplate
        const myProp = this.prop(prop);
        if (!myProp) throw new Error("<${name}> needs a valid prop");
        const matchCount = await myProp.trigger();
        if (matchCount === 0) throw new Error("<${name}> needs a prop with at least 1 match");
        //do something with the prop
    });
};
`);
}
function getContentTS(name) {
    return (`import {IAyakashiInstance, IDomProp} from "@ayakashi/types";

//action type definition, fill in actual type
declare module "@ayakashi/types/types/prelude/prelude" {
    export interface IAyakashiInstance {
        ${name}: (prop: IDomProp | string) => Promise<void>;
    }
}

export default function(ayakashi: IAyakashiInstance) {
    ayakashi.registerAction("${name}", async function(prop: IDomProp | string) {
        //prop boilerplate
        const myProp = this.prop(prop);
        if (!myProp) throw new Error("<${name}> needs a valid prop");
        const matchCount = await myProp.trigger();
        if (matchCount === 0) throw new Error("<${name}> needs a prop with at least 1 match");
        //do something with the prop
    });
}
`);
}
